@@ -93,7 +93,7 @@ class TestFileJpeg(PillowTestCase):
         self.assertEqual(test(72), (72, 72))
         self.assertEqual(test(300), (300, 300))
         self.assertEqual(test(100, 200), (100, 200))
-        self.assertEqual(test(0), None)  # square pixels
+        self.assertIsNone(test(0))  # square pixels
 
     def test_icc(self):
         # Test ICC support
@@ -133,6 +133,19 @@ class TestFileJpeg(PillowTestCase):
         test(ImageFile.MAXBLOCK+1)  # full buffer block plus one byte
         test(ImageFile.MAXBLOCK*4+3)  # large block
 
+    def test_large_icc_meta(self):
+        # https://github.com/python-pillow/Pillow/issues/148
+        # Sometimes the meta data on the icc_profile block is bigger than
+        # Image.MAXBLOCK or the image size.
+        im = Image.open('Tests/images/icc_profile_big.jpg')
+        f = self.tempfile("temp.jpg")
+        icc_profile = im.info["icc_profile"]
+        try:
+            im.save(f, format='JPEG', progressive=True,quality=95,
+                    icc_profile=icc_profile, optimize=True)
+        except IOError:
+            self.fail("Failed saving image with icc larger than image size")
+
     def test_optimize(self):
         im1 = self.roundtrip(hopper())
         im2 = self.roundtrip(hopper(), optimize=0)
@@ -439,7 +452,7 @@ class TestFileJpeg(PillowTestCase):
     def test_no_duplicate_0x1001_tag(self):
         # Arrange
         from PIL import ExifTags
-        tag_ids = dict(zip(ExifTags.TAGS.values(), ExifTags.TAGS.keys()))
+        tag_ids = {v: k for k, v in ExifTags.TAGS.items()}
 
         # Assert
         self.assertEqual(tag_ids['RelatedImageWidth'], 0x1001)
@@ -476,17 +489,11 @@ class TestFileJpeg(PillowTestCase):
             img.save(out, "JPEG")
 
     def test_save_wrong_modes(self):
-        out = BytesIO()
-        for mode in ['LA', 'La', 'RGBa', 'P']:
-            img = Image.new(mode, (20, 20))
-            self.assertRaises(IOError, img.save, out, "JPEG")
-
-    def test_save_modes_with_warnings(self):
         # ref https://github.com/python-pillow/Pillow/issues/2005
         out = BytesIO()
-        for mode in ['RGBA']:
+        for mode in ['LA', 'La', 'RGBA', 'RGBa', 'P']:
             img = Image.new(mode, (20, 20))
-            self.assert_warning(DeprecationWarning, img.save, out, "JPEG")
+            self.assertRaises(IOError, img.save, out, "JPEG")
 
     def test_save_tiff_with_dpi(self):
         # Arrange
@@ -501,14 +508,24 @@ class TestFileJpeg(PillowTestCase):
         reloaded.load()
         self.assertEqual(im.info['dpi'], reloaded.info['dpi'])
 
-    def test_dpi_from_exif(self):
+    def test_dpi_tuple_from_exif(self):
         # Arrange
         # This Photoshop CC 2017 image has DPI in EXIF not metadata
+        # EXIF XResolution is (2000000, 10000)
         im = Image.open("Tests/images/photoshop-200dpi.jpg")
 
         # Act / Assert
         self.assertEqual(im.info.get("dpi"), (200, 200))
 
+    def test_dpi_int_from_exif(self):
+        # Arrange
+        # This image has DPI in EXIF not metadata
+        # EXIF XResolution is 72
+        im = Image.open("Tests/images/exif-72dpi-int.jpg")
+
+        # Act / Assert
+        self.assertEqual(im.info.get("dpi"), (72, 72))
+
     def test_dpi_from_dpcm_exif(self):
         # Arrange
         # This is photoshop-200dpi.jpg with EXIF resolution unit set to cm:
@@ -535,7 +552,7 @@ class TestFileCloseW32(PillowTestCase):
     def setUp(self):
         if "jpeg_encoder" not in codecs or "jpeg_decoder" not in codecs:
             self.skipTest("jpeg support not available")
-            
+
     def test_fd_leak(self):
         tmpfile = self.tempfile("temp.jpg")
         import os
@@ -549,8 +566,9 @@ class TestFileCloseW32(PillowTestCase):
         self.assertRaises(Exception, lambda: os.remove(tmpfile))
         im.load()
         self.assertTrue(fp.closed)
-        # this should not fail, as load should have closed the file. 
+        # this should not fail, as load should have closed the file.
         os.remove(tmpfile)
 
+
 if __name__ == '__main__':
     unittest.main()
