@@ -42,6 +42,7 @@ class LibTiffTestCase(PillowTestCase):
         out_bytes = io.BytesIO()
         im.save(out_bytes, format='tiff', compression='group4')
 
+
 class TestFileLibTiff(LibTiffTestCase):
 
     def test_g4_tiff(self):
@@ -172,8 +173,7 @@ class TestFileLibTiff(LibTiffTestCase):
                                 'RowsPerStrip',
                                 'StripOffsets']
             for field in requested_fields:
-                self.assertTrue(field in reloaded,
-                                "%s not in metadata" % field)
+                self.assertIn(field, reloaded, "%s not in metadata" % field)
 
     def test_additional_metadata(self):
         # these should not crash. Seriously dummy data, most of it doesn't make
@@ -189,7 +189,7 @@ class TestFileLibTiff(LibTiffTestCase):
         # Exclude ones that have special meaning
         # that we're already testing them
         im = Image.open('Tests/images/hopper_g4.tif')
-        for tag in im.tag_v2.keys():
+        for tag in im.tag_v2:
             try:
                 del(core_items[tag])
             except:
@@ -532,7 +532,7 @@ class TestFileLibTiff(LibTiffTestCase):
         count = im.n_frames
         im.close()
         try:
-            os.remove(tmpfile) # Windows PermissionError here!
+            os.remove(tmpfile)  # Windows PermissionError here!
         except:
             self.fail("Should not get permission error here")
 
