@@ -1,10 +1,12 @@
-from __future__ import print_function
 import logging
+from io import BytesIO
 import struct
+import sys
 
 from helper import unittest, PillowTestCase, hopper, py3
 
 from PIL import Image, TiffImagePlugin
+from PIL.TiffImagePlugin import X_RESOLUTION, Y_RESOLUTION, RESOLUTION_UNIT
 
 logger = logging.getLogger(__name__)
 
@@ -39,7 +41,7 @@ class TestFileTiff(PillowTestCase):
         im = Image.open(filename)
 
     def test_mac_tiff(self):
-        # Read RGBa images from Mac OS X [@PIL136]
+        # Read RGBa images from macOS [@PIL136]
 
         filename = "Tests/images/pil136.tiff"
         im = Image.open(filename)
@@ -76,7 +78,6 @@ class TestFileTiff(PillowTestCase):
         self.assertEqual(im.mode, 'RGB')
 
     def test_xyres_tiff(self):
-        from PIL.TiffImagePlugin import X_RESOLUTION, Y_RESOLUTION
         filename = "Tests/images/pil168.tif"
         im = Image.open(filename)
 
@@ -84,14 +85,32 @@ class TestFileTiff(PillowTestCase):
         self.assertIsInstance(im.tag[X_RESOLUTION][0], tuple)
         self.assertIsInstance(im.tag[Y_RESOLUTION][0], tuple)
 
-        #v2 api
-        self.assertIsInstance(im.tag_v2[X_RESOLUTION], TiffImagePlugin.IFDRational)
-        self.assertIsInstance(im.tag_v2[Y_RESOLUTION], TiffImagePlugin.IFDRational)
+        # v2 api
+        self.assertIsInstance(im.tag_v2[X_RESOLUTION],
+                              TiffImagePlugin.IFDRational)
+        self.assertIsInstance(im.tag_v2[Y_RESOLUTION],
+                              TiffImagePlugin.IFDRational)
 
         self.assertEqual(im.info['dpi'], (72., 72.))
 
+    def test_xyres_fallback_tiff(self):
+        filename = "Tests/images/compression.tif"
+        im = Image.open(filename)
+
+        # v2 api
+        self.assertIsInstance(im.tag_v2[X_RESOLUTION],
+                              TiffImagePlugin.IFDRational)
+        self.assertIsInstance(im.tag_v2[Y_RESOLUTION],
+                              TiffImagePlugin.IFDRational)
+        self.assertRaises(KeyError,
+                          lambda: im.tag_v2[RESOLUTION_UNIT])
+
+        # Legacy.
+        self.assertEqual(im.info['resolution'], (100., 100.))
+        # Fallback "inch".
+        self.assertEqual(im.info['dpi'], (100., 100.))
+
     def test_int_resolution(self):
-        from PIL.TiffImagePlugin import X_RESOLUTION, Y_RESOLUTION
         filename = "Tests/images/pil168.tif"
         im = Image.open(filename)
 
@@ -101,13 +120,21 @@ class TestFileTiff(PillowTestCase):
         im._setup()
         self.assertEqual(im.info['dpi'], (71., 71.))
 
+    def test_save_setting_missing_resolution(self):
+        b = BytesIO()
+        Image.open("Tests/images/10ct_32bit_128.tiff").save(
+            b, format="tiff", resolution=123.45)
+        im = Image.open(b)
+        self.assertEqual(float(im.tag_v2[X_RESOLUTION]), 123.45)
+        self.assertEqual(float(im.tag_v2[Y_RESOLUTION]), 123.45)
+
     def test_invalid_file(self):
         invalid_file = "Tests/images/flower.jpg"
 
         self.assertRaises(SyntaxError,
                           lambda: TiffImagePlugin.TiffImageFile(invalid_file))
 
-        TiffImagePlugin.PREFIXES.append("\xff\xd8\xff\xe0")
+        TiffImagePlugin.PREFIXES.append(b"\xff\xd8\xff\xe0")
         self.assertRaises(SyntaxError,
                           lambda: TiffImagePlugin.TiffImageFile(invalid_file))
         TiffImagePlugin.PREFIXES.pop()
@@ -207,7 +234,7 @@ class TestFileTiff(PillowTestCase):
                 im.seek(n_frames)
                 break
             except EOFError:
-                self.assertTrue(im.tell() < n_frames)
+                self.assertLess(im.tell(), n_frames)
 
     def test_multipage(self):
         # issue #862
@@ -244,31 +271,30 @@ class TestFileTiff(PillowTestCase):
         # Assert
         self.assertIsInstance(ret, str)
 
-    def test_as_dict(self):
+    def test_dict(self):
         # Arrange
         filename = "Tests/images/pil136.tiff"
         im = Image.open(filename)
+
         # v2 interface
-        self.assertEqual(
-            im.tag_v2.as_dict(),
-            {256: 55, 257: 43, 258: (8, 8, 8, 8), 259: 1,
-             262: 2, 296: 2, 273: (8,), 338: (1,), 277: 4,
-             279: (9460,), 282: 72.0, 283: 72.0, 284: 1})
+        v2_tags = {256: 55, 257: 43, 258: (8, 8, 8, 8), 259: 1,
+                   262: 2, 296: 2, 273: (8,), 338: (1,), 277: 4,
+                   279: (9460,), 282: 72.0, 283: 72.0, 284: 1}
+        self.assertEqual(dict(im.tag_v2), v2_tags)
 
         # legacy interface
-        self.assertEqual(
-            im.tag.as_dict(),
-            {256: (55,), 257: (43,), 258: (8, 8, 8, 8), 259: (1,),
-             262: (2,), 296: (2,), 273: (8,), 338: (1,), 277: (4,),
-             279: (9460,), 282: ((720000, 10000),),
-             283: ((720000, 10000),), 284: (1,)})
+        legacy_tags = {256: (55,), 257: (43,), 258: (8, 8, 8, 8), 259: (1,),
+                       262: (2,), 296: (2,), 273: (8,), 338: (1,), 277: (4,),
+                       279: (9460,), 282: ((720000, 10000),),
+                       283: ((720000, 10000),), 284: (1,)}
+        self.assertEqual(dict(im.tag), legacy_tags)
 
     def test__delitem__(self):
         filename = "Tests/images/pil136.tiff"
         im = Image.open(filename)
-        len_before = len(im.ifd.as_dict())
+        len_before = len(dict(im.ifd))
         del im.ifd[256]
-        len_after = len(im.ifd.as_dict())
+        len_after = len(dict(im.ifd))
         self.assertEqual(len_before, len_after + 1)
 
     def test_load_byte(self):
@@ -276,7 +302,7 @@ class TestFileTiff(PillowTestCase):
             ifd = TiffImagePlugin.ImageFileDirectory_v2()
             data = b"abc"
             ret = ifd.load_byte(data, legacy_api)
-            self.assertEqual(ret, b"abc" if legacy_api else (97, 98, 99))
+            self.assertEqual(ret, b"abc")
 
     def test_load_string(self):
         ifd = TiffImagePlugin.ImageFileDirectory_v2()
@@ -328,19 +354,38 @@ class TestFileTiff(PillowTestCase):
         self.assertEqual(im.mode, "L")
         self.assert_image_similar(im, original, 7.3)
 
-    def test_page_number_x_0(self):
-        # Issue 973
-        # Test TIFF with tag 297 (Page Number) having value of 0 0.
-        # The first number is the current page number.
-        # The second is the total number of pages, zero means not available.
-        outfile = self.tempfile("temp.tif")
-        # Created by printing a page in Chrome to PDF, then:
-        # /usr/bin/gs -q -sDEVICE=tiffg3 -sOutputFile=total-pages-zero.tif
-        # -dNOPAUSE /tmp/test.pdf -c quit
-        infile = "Tests/images/total-pages-zero.tif"
-        im = Image.open(infile)
-        # Should not divide by zero
-        im.save(outfile)
+    def test_gray_semibyte_per_pixel(self):
+        test_files = (
+            (
+                24.8,  # epsilon
+                (  # group
+                    "Tests/images/tiff_gray_2_4_bpp/hopper2.tif",
+                    "Tests/images/tiff_gray_2_4_bpp/hopper2I.tif",
+                    "Tests/images/tiff_gray_2_4_bpp/hopper2R.tif",
+                    "Tests/images/tiff_gray_2_4_bpp/hopper2IR.tif",
+                )
+            ),
+            (
+                7.3,  # epsilon
+                (  # group
+                    "Tests/images/tiff_gray_2_4_bpp/hopper4.tif",
+                    "Tests/images/tiff_gray_2_4_bpp/hopper4I.tif",
+                    "Tests/images/tiff_gray_2_4_bpp/hopper4R.tif",
+                    "Tests/images/tiff_gray_2_4_bpp/hopper4IR.tif",
+                )
+            ),
+        )
+        original = hopper("L")
+        for epsilon, group in test_files:
+            im = Image.open(group[0])
+            self.assertEqual(im.size, (128, 128))
+            self.assertEqual(im.mode, "L")
+            self.assert_image_similar(im, original, epsilon)
+            for file in group[1:]:
+                im2 = Image.open(file)
+                self.assertEqual(im2.size, (128, 128))
+                self.assertEqual(im2.mode, "L")
+                self.assert_image_equal(im, im2)
 
     def test_with_underscores(self):
         kwargs = {'resolution_unit': 'inch',
@@ -359,43 +404,120 @@ class TestFileTiff(PillowTestCase):
         self.assertEqual(im.tag_v2[X_RESOLUTION], 72)
         self.assertEqual(im.tag_v2[Y_RESOLUTION], 36)
 
-    def test_deprecation_warning_with_spaces(self):
-        kwargs = {'resolution unit': 'inch',
-                  'x resolution': 36,
-                  'y resolution': 72}
-        filename = self.tempfile("temp.tif")
-        self.assert_warning(DeprecationWarning,
-                            lambda: hopper("RGB").save(filename, **kwargs))
-        from PIL.TiffImagePlugin import X_RESOLUTION, Y_RESOLUTION
+    def test_lzw(self):
+        # Act
+        im = Image.open("Tests/images/hopper_lzw.tif")
 
-        im = Image.open(filename)
+        # Assert
+        self.assertEqual(im.mode, 'RGB')
+        self.assertEqual(im.size, (128, 128))
+        self.assertEqual(im.format, "TIFF")
+        im2 = hopper()
+        self.assert_image_similar(im, im2, 5)
 
-        # legacy interface
-        self.assertEqual(im.tag[X_RESOLUTION][0][0], 36)
-        self.assertEqual(im.tag[Y_RESOLUTION][0][0], 72)
+    def test_roundtrip_tiff_uint16(self):
+        # Test an image of all '0' values
+        pixel_value = 0x1234
+        infile = "Tests/images/uint16_1_4660.tif"
+        im = Image.open(infile)
+        self.assertEqual(im.getpixel((0, 0)), pixel_value)
 
-        # v2 interface
-        self.assertEqual(im.tag_v2[X_RESOLUTION], 36)
-        self.assertEqual(im.tag_v2[Y_RESOLUTION], 72)
+        tmpfile = self.tempfile("temp.tif")
+        im.save(tmpfile)
 
-    def test_multipage_compression(self):
-        im = Image.open('Tests/images/compression.tif')
+        reloaded = Image.open(tmpfile)
 
-        im.seek(0)
-        self.assertEqual(im._compression,'tiff_ccitt')
-        self.assertEqual(im.size, (10, 10))
+        self.assert_image_equal(im, reloaded)
 
-        im.seek(1)
-        self.assertEqual(im._compression,'packbits')
-        self.assertEqual(im.size, (10, 10))
+    def test_tiff_save_all(self):
+        import io
+        import os
+
+        mp = io.BytesIO()
+        with Image.open("Tests/images/multipage.tiff") as im:
+            im.save(mp, format="tiff", save_all=True)
+
+        mp.seek(0, os.SEEK_SET)
+        with Image.open(mp) as im:
+            self.assertEqual(im.n_frames, 3)
+
+        # Test appending images
+        mp = io.BytesIO()
+        im = Image.new('RGB', (100, 100), '#f00')
+        ims = [Image.new('RGB', (100, 100), color) for color
+               in ['#0f0', '#00f']]
+        im.save(mp, format="TIFF", save_all=True, append_images=ims)
+
+        mp.seek(0, os.SEEK_SET)
+        reread = Image.open(mp)
+        self.assertEqual(reread.n_frames, 3)
+
+
+    def test_saving_icc_profile(self):
+        # Tests saving TIFF with icc_profile set.
+        # At the time of writing this will only work for non-compressed tiffs
+        # as libtiff does not support embedded ICC profiles,
+        # ImageFile._save(..) however does.
+        im = Image.new('RGB', (1, 1))
+        im.info['icc_profile'] = 'Dummy value'
+
+        # Try save-load round trip to make sure both handle icc_profile.
+        tmpfile = self.tempfile('temp.tif')
+        im.save(tmpfile, 'TIFF', compression='raw')
+        reloaded = Image.open(tmpfile)
+
+        self.assertEqual(b'Dummy value', reloaded.info['icc_profile'])
+
+    def test_close_on_load_exclusive(self):
+        # similar to test_fd_leak, but runs on unixlike os
+        tmpfile = self.tempfile("temp.tif")
+
+        with Image.open("Tests/images/uint16_1_4660.tif") as im:
+            im.save(tmpfile)
+
+        im = Image.open(tmpfile)
+        fp = im.fp
+        self.assertFalse(fp.closed)
         im.load()
+        self.assertTrue(fp.closed)
 
-        im.seek(0)
-        self.assertEqual(im._compression,'tiff_ccitt')
-        self.assertEqual(im.size, (10, 10))
+    def test_close_on_load_nonexclusive(self):
+        tmpfile = self.tempfile("temp.tif")
+
+        with Image.open("Tests/images/uint16_1_4660.tif") as im:
+            im.save(tmpfile)
+
+        f = open(tmpfile, 'rb')
+        im = Image.open(f)
+        fp = im.fp
+        self.assertFalse(fp.closed)
         im.load()
+        self.assertFalse(fp.closed)
+
+@unittest.skipUnless(sys.platform.startswith('win32'), "Windows only")
+class TestFileTiffW32(PillowTestCase):
+    def test_fd_leak(self):
+        tmpfile = self.tempfile("temp.tif")
+        import os
+
+        # this is an mmaped file.
+        with Image.open("Tests/images/uint16_1_4660.tif") as im:
+            im.save(tmpfile)
+
+        im = Image.open(tmpfile)
+        fp = im.fp
+        self.assertFalse(fp.closed)
+        self.assertRaises(Exception, lambda: os.remove(tmpfile))
+        im.load()
+        self.assertTrue(fp.closed)
+
+        # this closes the mmap
+        im.close()
+
+        # this should not fail, as load should have closed the file pointer,
+        # and close should have closed the mmap
+        os.remove(tmpfile)
+
 
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
