@@ -1,6 +1,6 @@
-from helper import unittest, PillowTestCase
+from helper import unittest, PillowTestCase, hopper
 from PIL import Image
-from io import BytesIO
+
 
 class TestFileWmf(PillowTestCase):
 
@@ -26,6 +26,13 @@ class TestFileWmf(PillowTestCase):
             imref.load()
             self.assert_image_similar(im, imref, 2.0)
 
+    def test_save(self):
+        im = hopper()
+
+        for ext in [".wmf", ".emf"]:
+            tmpfile = self.tempfile("temp"+ext)
+            self.assertRaises(IOError, lambda: im.save(tmpfile))
+
 
 if __name__ == '__main__':
     unittest.main()
