@@ -71,8 +71,6 @@
  * See the README file for information on usage and redistribution.
  */
 
-#define PILLOW_VERSION "4.1.0"
-
 #include "Python.h"
 
 #ifdef HAVE_LIBZ
@@ -96,8 +94,6 @@
 #define WITH_THREADING /* "friendly" threading support */
 #define WITH_UNSHARPMASK /* Kevin Cazabon's unsharpmask module */
 
-#define WITH_DEBUG /* extra debugging interfaces */
-
 #undef    VERBOSE
 
 #define CLIP(x) ((x) <= 0 ? 0 : (x) < 256 ? (x) : 255)
@@ -633,18 +629,6 @@ _new(PyObject* self, PyObject* args)
     return PyImagingNew(ImagingNew(mode, xsize, ysize));
 }
 
-static PyObject*
-_new_array(PyObject* self, PyObject* args)
-{
-    char* mode;
-    int xsize, ysize;
-
-    if (!PyArg_ParseTuple(args, "s(ii)", &mode, &xsize, &ysize))
-        return NULL;
-
-    return PyImagingNew(ImagingNewArray(mode, xsize, ysize));
-}
-
 static PyObject*
 _new_block(PyObject* self, PyObject* args)
 {
@@ -2922,8 +2906,6 @@ _getcodecstatus(PyObject* self, PyObject* args)
 /* -------------------------------------------------------------------- */
 
 
-#ifdef WITH_DEBUG
-
 static PyObject*
 _save_ppm(ImagingObject* self, PyObject* args)
 {
@@ -2939,7 +2921,6 @@ _save_ppm(ImagingObject* self, PyObject* args)
     return Py_None;
 }
 
-#endif
 
 /* -------------------------------------------------------------------- */
 
@@ -3032,12 +3013,9 @@ static struct PyMethodDef methods[] = {
 #endif
 
     /* Misc. */
-    {"new_array", (PyCFunction)_new_array, 1},
     {"new_block", (PyCFunction)_new_block, 1},
 
-#ifdef WITH_DEBUG
     {"save_ppm", (PyCFunction)_save_ppm, 1},
-#endif
 
     {NULL, NULL} /* sentinel */
 };
@@ -3437,6 +3415,7 @@ static PyMethodDef functions[] = {
 static int
 setup_module(PyObject* m) {
     PyObject* d = PyModule_GetDict(m);
+    const char* version = (char*)PILLOW_VERSION;
 
     /* Ready object types */
     if (PyType_Ready(&Imaging_Type) < 0)
@@ -3481,7 +3460,7 @@ setup_module(PyObject* m) {
   }
 #endif
 
-    PyDict_SetItemString(d, "PILLOW_VERSION", PyUnicode_FromString(PILLOW_VERSION));
+    PyDict_SetItemString(d, "PILLOW_VERSION", PyUnicode_FromString(version));
 
     return 0;
 }
